import logging
from functools import lru_cache
import re

from google.api_core import exceptions
from google.cloud import datacatalog
from google.cloud import datacatalog_v1beta1


class DataCatalogFacade:
    """Data Catalog API communication facade."""

    __NESTED_LOG_PREFIX = ' ' * 5

    def __init__(self):
        # Initialize the API client.
        self.__datacatalog = datacatalog.DataCatalogClient()

    def create_tag_template(self, project_id, location_id, tag_template_id, tag_template):
        return self.__datacatalog.create_tag_template(
            parent=datacatalog.DataCatalogClient.location_path(project_id, location_id),
            tag_template_id=tag_template_id,
            tag_template=tag_template)

    def delete_tag_template(self, name):
        self.__datacatalog.delete_tag_template(name=name, force=True)

    def list_tags(self, entry_name):
        return self.__list_tags(entry_name)

    # Currently we don't have a list method, so we are using search which is not exhaustive,
    # and might not return some entries.
    def search_tag_templates(self, project_ids):
        scope = datacatalog_v1beta1.types.SearchCatalogRequest.Scope()

        scope.include_project_ids.extend(project_ids.split(','))

        query = 'type=TAG_TEMPLATE'

        results_iterator = self.__datacatalog.search_catalog(scope=scope,
                                                             query=query,
                                                             order_by='relevance',
                                                             page_size=1000)

        results = []
        for page in results_iterator.pages:
            results.extend(page)

        return results

    # Currently we don't have a list method, so we are using search which is not exhaustive,
    # and might not return some entries.
    def search_tagged_assets(self, project_id, tag_template_id):
        scope = datacatalog_v1beta1.types.SearchCatalogRequest.Scope()

        scope.include_project_ids.extend([project_id])

        query = 'tag:{}.{}'.format(project_id, tag_template_id)

        results_iterator = self.__datacatalog.search_catalog(scope=scope,
                                                             query=query,
                                                             order_by='relevance',
                                                             page_size=1000)

        results = []
        for page in results_iterator.pages:
            results.extend(page)

        return results

    @lru_cache(maxsize=16)
    def get_tag_template(self, name):
        self.__log_operation_start('GET Tag Template: %s', name)
        tag_template = self.__datacatalog.get_tag_template(name=name)
        self.__log_single_object_read_result(tag_template)
        return tag_template

    def get_tag_templates_from_search_results(self, search_results):
        tag_templates = []

        for search_result in search_results:
            template_name = search_result.relative_resource_name
            try:
                tag_template = self.get_tag_template(template_name)
                tag_templates.append(tag_template)
            except exceptions.GoogleAPICallError as e:
                logging.warning('Exception getting Tag Template %s: %s', template_name, str(e))

        return tag_templates

    @classmethod
    def extract_resources_from_template(cls, tag_template_name):
        re_match = re.match(
            r'^projects[/]([_a-zA-Z-\d]+)[/]locations[/]'
            r'([a-zA-Z-\d]+)[/]tagTemplates[/]([@a-zA-Z-_\d]+)$', tag_template_name)

        if re_match:
            project_id, location_id, tag_template_id, = re_match.groups()
            return project_id, location_id, tag_template_id

    @lru_cache(maxsize=16)
    def __list_tags(self, entry_name):
        return [tag for tag in self.__datacatalog.list_tags(parent=entry_name)]

    @classmethod
    def __log_operation_start(cls, message, *args):
        logging.info('')
        logging.info(message, *args)
        logging.info('--------------------------------------------------')

    @classmethod
    def __log_single_object_read_result(cls, the_object):
        logging.info('%sFound!' if the_object else '%sNOT found!', cls.__NESTED_LOG_PREFIX)



import argparse
import logging
import sys

from datacatalog_fileset_enricher import datacatalog_fileset_enricher
from datacatalog_tag_manager import tag_datasource_processor
from datacatalog_util import tag_datasource_exporter
from datacatalog_util import tag_template_datasource_processor
from datacatalog_util import tag_template_datasource_exporter


class DatacatalogUtilsCLI:

    @classmethod
    def run(cls, argv):
        cls.__setup_logging()

        args = cls._parse_args(argv)
        args.func(args)

    @classmethod
    def __setup_logging(cls):
        logging.basicConfig(level=logging.INFO)

    @classmethod
    def _parse_args(cls, argv):
        parser = argparse.ArgumentParser(description=__doc__,
                                         formatter_class=argparse.RawDescriptionHelpFormatter)

        subparsers = parser.add_subparsers()

        cls.add_tag_templates_cmd(subparsers)

        cls.add_tags_cmd(subparsers)

        cls.add_fileset_enricher_cmd(subparsers)

        return parser.parse_args(argv)

    @classmethod
    def add_tags_cmd(cls, subparsers):
        tags_parser = subparsers.add_parser("tags", help="Tags commands")

        tags_subparsers = tags_parser.add_subparsers()

        cls.add_create_tags_cmd(tags_subparsers)

        cls.add_export_tags_cmd(tags_subparsers)

    @classmethod
    def add_tag_templates_cmd(cls, subparsers):
        tag_templates_parser = subparsers.add_parser("tag-templates",
                                                     help="Tag Templates commands")

        tag_templates_subparsers = tag_templates_parser.add_subparsers()

        cls.add_create_tag_templates_cmd(tag_templates_subparsers)

        cls.add_delete_tag_templates_cmd(tag_templates_subparsers)

        cls.add_export_tag_templates_cmd(tag_templates_subparsers)

    @classmethod
    def add_export_tags_cmd(cls, subparsers):
        export_tags_parser = subparsers.add_parser('export',
                                                   help='Export Tags to CSV,'
                                                   ' creates one file'
                                                   ' for each teamplate')
        export_tags_parser.add_argument('--dir-path',
                                        help='Directory path where files will be exported')
        export_tags_parser.add_argument('--project-ids',
                                        help='Project ids to narrow down Templates list,'
                                        'split by comma',
                                        required=True)
        export_tags_parser.set_defaults(func=cls.__export_tags)

    @classmethod
    def add_export_tag_templates_cmd(cls, subparsers):
        export_tag_templates_parser = subparsers.add_parser('export',
                                                            help='Export Tag Templates to CSV')
        export_tag_templates_parser.add_argument('--file-path',
                                                 help='File path where file will be exported')
        export_tag_templates_parser.add_argument('--project-ids',
                                                 help='Project ids to narrow down Templates list,'
                                                 'split by comma',
                                                 required=True)
        export_tag_templates_parser.set_defaults(func=cls.__export_tag_templates)

    @classmethod
    def add_fileset_enricher_cmd(cls, subparsers):
        filesets_parser = subparsers.add_parser("filesets", help="Filesets commands")

        filesets_parser.add_argument('--project-id', help='Project id', required=True)

        filesets_subparsers = filesets_parser.add_subparsers()

        enrich_filesets = filesets_subparsers.add_parser('enrich',
                                                         help='Enrich GCS filesets with Tags')

        enrich_filesets.add_argument('--entry-group-id', help='Entry Group ID')
        enrich_filesets.add_argument('--entry-id', help='Entry ID')
        enrich_filesets.add_argument('--tag-fields',
                                     help='Specify the fields you want on the generated Tags,'
                                     ' split by comma, use the list available in the docs')
        enrich_filesets.add_argument('--bucket-prefix',
                                     help='Specify a bucket prefix if you want to avoid scanning'
                                     ' too many GCS buckets')
        enrich_filesets.set_defaults(func=cls.__enrich_fileset)

        clean_up_tags = filesets_subparsers.add_parser(
            'clean-up-templates-and-tags',
            help='Clean up the Fileset Enhancer Template and Tags From the Fileset Entries')
        clean_up_tags.set_defaults(func=cls.__clean_up_fileset_template_and_tags)

        clean_up_all = filesets_subparsers.add_parser(
            'clean-up-all',
            help='Clean up Fileset Entries, Their Tags and the Fileset Enhancer Template')
        clean_up_all.set_defaults(func=cls.__clean_up_all_filesets)

    @classmethod
    def add_delete_tag_templates_cmd(cls, subparsers):
        delete_tag_templates_parser = subparsers.add_parser('delete',
                                                            help='Delete Tag Templates from CSV')
        delete_tag_templates_parser.add_argument('--csv-file',
                                                 help='CSV file with Tag Templates information',
                                                 required=True)
        delete_tag_templates_parser.set_defaults(func=cls.__delete_tag_templates)

    @classmethod
    def add_create_tag_templates_cmd(cls, subparsers):
        create_tag_templates_parser = subparsers.add_parser('create',
                                                            help='Create Tag Templates from CSV')
        create_tag_templates_parser.add_argument('--csv-file',
                                                 help='CSV file with Tag Templates information',
                                                 required=True)
        create_tag_templates_parser.set_defaults(func=cls.__create_tag_templates)

    @classmethod
    def add_create_tags_cmd(cls, subparsers):
        create_tags_parser = subparsers.add_parser('create', help='Create Tags from CSV')
        create_tags_parser.add_argument('--csv-file',
                                        help='CSV file with Tags information',
                                        required=True)
        create_tags_parser.set_defaults(func=cls.__create_tags)

    @classmethod
    def __enrich_fileset(cls, args):
        tag_fields = None
        if args.tag_fields:
            tag_fields = args.tag_fields.split(',')

        datacatalog_fileset_enricher.DatacatalogFilesetEnricher(args.project_id).run(
            args.entry_group_id, args.entry_id, tag_fields, args.bucket_prefix)

    @classmethod
    def __clean_up_fileset_template_and_tags(cls, args):
        datacatalog_fileset_enricher.DatacatalogFilesetEnricher(
            args.project_id).clean_up_fileset_template_and_tags()

    @classmethod
    def __clean_up_all_filesets(cls, args):
        datacatalog_fileset_enricher.DatacatalogFilesetEnricher(args.project_id).clean_up_all()

    @classmethod
    def __create_tags(cls, args):
        tag_datasource_processor.TagDatasourceProcessor().create_tags_from_csv(
            file_path=args.csv_file)

    @classmethod
    def __create_tag_templates(cls, args):
        tag_template_datasource_processor.TagTemplateDatasourceProcessor(
        ).create_tag_templates_from_csv(file_path=args.csv_file)

    @classmethod
    def __delete_tag_templates(cls, args):
        tag_template_datasource_processor.TagTemplateDatasourceProcessor(
        ).delete_tag_templates_from_csv(file_path=args.csv_file)

    @classmethod
    def __export_tag_templates(cls, args):
        tag_template_datasource_exporter.TagTemplateDatasourceExporter().export_tag_templates(
            project_ids=args.project_ids, file_path=args.file_path)

    @classmethod
    def __export_tags(cls, args):
        tag_datasource_exporter.TagDatasourceExporter().export_tags(project_ids=args.project_ids,
                                                                    dir_path=args.dir_path)


def main():
    argv = sys.argv
    DatacatalogUtilsCLI.run(argv[1:] if len(argv) > 0 else argv)


if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



"""Top-level package for bq-properties-creator."""

from .datacatalog_util_cli import main  # noqa: F401

__author__ = """Marcelo Miranda"""
__email__ = 'mesmacosta@gmail.com'
__version__ = '0.2.0'

__all__ = ('main')



import unittest

from google.cloud.datacatalog import enums

from datacatalog_util import datacatalog_entity_factory


class DataCatalogEntityFactoryTest(unittest.TestCase):
    __BOOL_TYPE = enums.FieldType.PrimitiveType.BOOL
    __DOUBLE_TYPE = enums.FieldType.PrimitiveType.DOUBLE
    __STRING_TYPE = enums.FieldType.PrimitiveType.STRING
    __TIMESTAMP_TYPE = enums.FieldType.PrimitiveType.TIMESTAMP

    def test_make_tag_template_valid_boolean_values_should_set_fields(self):

        tag_template_dict = {
            'display_name': 'My Tag Template',
            'fields': {
                'my_bool_field': {
                    'field_display_name': 'My BOOL field',
                    'field_type': 'BOOL'
                }
            }
        }

        tag_template = datacatalog_entity_factory.DataCatalogEntityFactory.make_tag_template(
            tag_template_dict)

        my_bool_field = tag_template.fields['my_bool_field']

        self.assertEqual(tag_template_dict['display_name'], tag_template.display_name)
        self.assertEqual(tag_template_dict['fields']['my_bool_field']['field_display_name'],
                         my_bool_field.display_name)
        self.assertEqual(self.__BOOL_TYPE, my_bool_field.type.primitive_type)

    def test_make_tag_template_valid_double_values_should_set_fields(self):

        tag_template_dict = {
            'display_name': 'My Tag Template',
            'fields': {
                'my_double_field': {
                    'field_display_name': 'My DOUBLE field',
                    'field_type': 'DOUBLE'
                }
            }
        }

        tag_template = datacatalog_entity_factory.DataCatalogEntityFactory.make_tag_template(
            tag_template_dict)

        my_double_field = tag_template.fields['my_double_field']

        self.assertEqual(tag_template_dict['display_name'], tag_template.display_name)
        self.assertEqual(tag_template_dict['fields']['my_double_field']['field_display_name'],
                         my_double_field.display_name)
        self.assertEqual(self.__DOUBLE_TYPE, my_double_field.type.primitive_type)

    def test_make_tag_template_valid_string_values_should_set_fields(self):

        tag_template_dict = {
            'display_name': 'My Tag Template',
            'fields': {
                'my_string_field': {
                    'field_display_name': 'My STRING field',
                    'field_type': 'STRING'
                }
            }
        }

        tag_template = datacatalog_entity_factory.DataCatalogEntityFactory.make_tag_template(
            tag_template_dict)

        my_string_field = tag_template.fields['my_string_field']

        self.assertEqual(tag_template_dict['display_name'], tag_template.display_name)
        self.assertEqual(tag_template_dict['fields']['my_string_field']['field_display_name'],
                         my_string_field.display_name)
        self.assertEqual(self.__STRING_TYPE, my_string_field.type.primitive_type)

    def test_make_tag_template_valid_timestamp_values_should_set_fields(self):

        tag_template_dict = {
            'display_name': 'My Tag Template',
            'fields': {
                'my_timestamp_field': {
                    'field_display_name': 'My TIMESTAMP field',
                    'field_type': 'TIMESTAMP'
                }
            }
        }

        tag_template = datacatalog_entity_factory.DataCatalogEntityFactory.make_tag_template(
            tag_template_dict)

        my_timestamp_field = tag_template.fields['my_timestamp_field']

        self.assertEqual(tag_template_dict['display_name'], tag_template.display_name)
        self.assertEqual(tag_template_dict['fields']['my_timestamp_field']['field_display_name'],
                         my_timestamp_field.display_name)
        self.assertEqual(self.__TIMESTAMP_TYPE, my_timestamp_field.type.primitive_type)

    def test_make_tag_template_valid_enum_values_should_set_fields(self):

        tag_template_dict = {
            'display_name': 'My Tag Template',
            'fields': {
                'my_enum_field': {
                    'field_display_name': 'My ENUM field',
                    'field_type': 'ENUM',
                    'enum_values': ['1', '2']
                }
            }
        }

        tag_template = datacatalog_entity_factory.DataCatalogEntityFactory.make_tag_template(
            tag_template_dict)

        my_enum_field = tag_template.fields['my_enum_field']

        self.assertEqual(tag_template_dict['display_name'], tag_template.display_name)
        self.assertEqual(tag_template_dict['fields']['my_enum_field']['field_display_name'],
                         my_enum_field.display_name)
        self.assertEqual(tag_template_dict['fields']['my_enum_field']['enum_values'][0],
                         my_enum_field.type.enum_type.allowed_values[0].display_name)
        self.assertEqual(tag_template_dict['fields']['my_enum_field']['enum_values'][1],
                         my_enum_field.type.enum_type.allowed_values[1].display_name)



import unittest
from unittest import mock

import datacatalog_tag_manager
from datacatalog_util import datacatalog_util_cli


class TagManagerCLITest(unittest.TestCase):

    def test_parse_args_invalid_subcommand_should_raise_system_exit(self):
        self.assertRaises(SystemExit, datacatalog_util_cli.DatacatalogUtilsCLI._parse_args,
                          ['invalid-subcommand'])

    def test_parse_args_create_tags_missing_mandatory_args_should_raise_system_exit(self):
        self.assertRaises(SystemExit, datacatalog_util_cli.DatacatalogUtilsCLI._parse_args,
                          ['tags', 'create'])

    def test_parse_args_create_tags_should_parse_mandatory_args(self):
        args = datacatalog_util_cli.DatacatalogUtilsCLI._parse_args(
            ['tags', 'create', '--csv-file', 'test.csv'])
        self.assertEqual('test.csv', args.csv_file)

    def test_run_no_args_should_raise_attribute_error(self):
        self.assertRaises(AttributeError, datacatalog_util_cli.DatacatalogUtilsCLI.run, None)

    @mock.patch(
        'datacatalog_util.datacatalog_util_cli.tag_datasource_processor.TagDatasourceProcessor')
    def test_run_create_tags_should_call_tag_creator(self, mock_tag_datasource_processor):
        datacatalog_util_cli.DatacatalogUtilsCLI.run(['tags', 'create', '--csv-file', 'test.csv'])

        tag_datasource_processor = mock_tag_datasource_processor.return_value
        tag_datasource_processor.create_tags_from_csv.assert_called_once()
        tag_datasource_processor.create_tags_from_csv.assert_called_with(file_path='test.csv')

    @mock.patch('datacatalog_util.datacatalog_util_cli.tag_template_datasource_processor.'
                'TagTemplateDatasourceProcessor')
    def test_run_create_tag_templates_should_call_tag_template_creator(
        self, mock_tag_template_datasource_processor):  # noqa: E125

        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['tag-templates', 'create', '--csv-file', 'test.csv'])

        tag_template_datasource_processor = mock_tag_template_datasource_processor.return_value
        tag_template_datasource_processor.create_tag_templates_from_csv.assert_called_once()
        tag_template_datasource_processor.create_tag_templates_from_csv.assert_called_with(
            file_path='test.csv')

    @mock.patch('datacatalog_util.datacatalog_util_cli.tag_template_datasource_processor.'
                'TagTemplateDatasourceProcessor')
    def test_run_delete_tag_templates_should_call_correct_method(
        self, mock_tag_template_datasource_processor):  # noqa: E125

        datacatalog_util_cli.DatacatalogUtilsCLI.run(
            ['tag-templates', 'delete', '--csv-file', 'test.csv'])

        tag_template_datasource_processor = mock_tag_template_datasource_processor.return_value
        tag_template_datasource_processor.delete_tag_templates_from_csv.assert_called_once()
        tag_template_datasource_processor.delete_tag_templates_from_csv.assert_called_with(
            file_path='test.csv')

    @mock.patch('datacatalog_util.datacatalog_util_cli.tag_template_datasource_exporter.'
                'TagTemplateDatasourceExporter')
    def test_run_export_tag_templates_should_call_correct_method(
        self, mock_tag_template_datasource_exporter):  # noqa: E125

        datacatalog_util_cli.DatacatalogUtilsCLI.run([
            'tag-templates', 'export', '--file-path', 'test.csv', '--project-ids',
            'my-project1,my-project2'
        ])

        tag_template_datasource_processor = mock_tag_template_datasource_exporter.return_value
        tag_template_datasource_processor.export_tag_templates.assert_called_once()
        tag_template_datasource_processor.export_tag_templates.assert_called_with(
            project_ids='my-project1,my-project2', file_path='test.csv')

    @mock.patch(
        'datacatalog_util.datacatalog_util_cli.tag_datasource_exporter.TagDatasourceExporter')
    def test_run_export_tags_should_call_correct_method(self, mock_tag_datasource_exporter):
        datacatalog_util_cli.DatacatalogUtilsCLI.run([
            'tags', 'export', '--dir-path', 'test.csv', '--project-ids', 'my-project1,my-project2'
        ])

        tag_datasource_processor = mock_tag_datasource_exporter.return_value
        tag_datasource_processor.export_tags.assert_called_once()
        tag_datasource_processor.export_tags.assert_called_with(
            project_ids='my-project1,my-project2', dir_path='test.csv')

    @mock.patch('datacatalog_tag_manager.tag_manager_cli.TagManagerCLI')
    def test_main_should_call_cli_run(self, mock_cli):
        datacatalog_tag_manager.main()
        mock_cli.run.assert_called_once()



from google.cloud import datacatalog_v1beta1


class DataCatalogEntityFactory:
    __TRUTHS = {1, '1', 't', 'T', 'true', 'True', 'TRUE'}

    @classmethod
    def make_tag_template(cls, tag_template_dict):
        tag_template = datacatalog_v1beta1.types.TagTemplate()

        tag_template.display_name = tag_template_dict['display_name']

        fields = tag_template_dict['fields']

        for field_id, items in fields.items():
            field_display_name = items['field_display_name']
            field_type = items['field_type']

            tag_template.fields[field_id].display_name = field_display_name

            if field_type == 'BOOL':
                tag_template.fields[field_id].type.primitive_type = \
                    datacatalog_v1beta1.enums.FieldType.PrimitiveType.BOOL.value
            elif field_type == 'DOUBLE':
                tag_template.fields[field_id].type.primitive_type = \
                    datacatalog_v1beta1.enums.FieldType.PrimitiveType.DOUBLE.value
            elif field_type == 'STRING':
                tag_template.fields[field_id].type.primitive_type = \
                    datacatalog_v1beta1.enums.FieldType.PrimitiveType.STRING.value
            elif field_type == 'TIMESTAMP':
                tag_template.fields[field_id].type.primitive_type = \
                    datacatalog_v1beta1.enums.FieldType.PrimitiveType.TIMESTAMP.value
            elif field_type == 'ENUM':
                enum_values = items['enum_values']
                for enum_value in enum_values:
                    tag_template.fields[field_id].type.enum_type \
                        .allowed_values.add().display_name = enum_value

        return tag_template

    @classmethod
    def get_primitive_field_type_for(cls, primitive_type):
        return datacatalog_v1beta1.enums.FieldType.PrimitiveType(primitive_type).name



import logging
import os

import pandas as pd

from . import constant, datacatalog_entity_factory, datacatalog_facade


class TagTemplateDatasourceExporter:

    def __init__(self):
        self.__datacatalog_facade = datacatalog_facade.DataCatalogFacade()

    def export_tag_templates(self, project_ids, file_path=None):
        """
        Export Tag Templates found by searching Data Catalog.

        :param file_path: File path to be exported to.
        :param project_ids: Project ids to narrow down search results.
        """
        logging.info('')
        logging.info('===> Export Tag Templates [STARTED]')

        logging.info('')
        logging.info('Exporting the Tag Templates...')
        self.__export_tag_templates(project_ids, file_path)

        logging.info('')
        logging.info('==== Export Tag Templates [FINISHED] =============')

    def __export_tag_templates(self, project_ids, file_path=None):
        search_results = self.__datacatalog_facade.search_tag_templates(project_ids)
        tag_templates = self.__datacatalog_facade.get_tag_templates_from_search_results(
            search_results)
        dataframe = self.__tag_templates_to_dataframe(tag_templates)

        if file_path is None:
            file_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'templates.csv')

        dataframe.to_csv(file_path)
        logging.info('Check the generated file at: %s', file_path)

    @classmethod
    def __tag_templates_to_dataframe(cls, tag_templates):
        dataframe = pd.DataFrame(columns=constant.TAG_TEMPLATES_DS_COLUMNS_ORDER)

        for tag_template in tag_templates:
            tag_template_name = tag_template.name
            tag_template_display_name = tag_template.display_name

            fields = tag_template.fields

            for field_id, field_values in fields.items():
                field_display_name = field_values.display_name
                field_type = field_values.type
                allowed_values = field_type.enum_type.allowed_values
                enum_values = ''
                if len(field_type.enum_type.allowed_values) > 0:
                    enum_values = constant.ENUM_VALUES_SEPARATOR.join(
                        [allowed_value.display_name for allowed_value in allowed_values])
                    field_type_str = 'ENUM'
                else:
                    field_type_str = datacatalog_entity_factory.DataCatalogEntityFactory.\
                        get_primitive_field_type_for(field_type.primitive_type)

                field_index = list(fields.keys()).index(field_id)

                if field_index == 0:
                    dataframe = dataframe.append(
                        {
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[0]: tag_template_name,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[1]: tag_template_display_name,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[2]: field_id,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[3]: field_display_name,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[4]: field_type_str,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[5]: enum_values
                        },
                        ignore_index=True)
                else:
                    dataframe = dataframe.append(
                        {
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[0]: '',
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[1]: '',
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[2]: field_id,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[3]: field_display_name,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[4]: field_type_str,
                            constant.TAG_TEMPLATES_DS_COLUMNS_ORDER[5]: enum_values
                        },
                        ignore_index=True)

        dataframe.set_index(constant.TAG_TEMPLATES_DS_TEMPLATE_NAME_COLUMN_LABEL, inplace=True)

        return dataframe



# Constants used for Tagging the informed Datasource, Tags Table.
TAGS_DS_FIELD_ID_COLUMN_LABEL = 'field_id'
TAGS_DS_FIELD_TYPE_COLUMN_LABEL = 'field_type'
TAGS_DS_FIELD_VALUE_COLUMN_LABEL = 'field_value'
TAGS_DS_LINKED_RESOURCE_COLUMN_LABEL = 'linked_resource'
TAGS_DS_RELATIVE_RESOURCE_NAME_COLUMN_LABEL = 'relative_resource_name'
TAGS_DS_SCHEMA_COLUMN_COLUMN_LABEL = 'column'
TAGS_DS_TEMPLATE_NAME_COLUMN_LABEL = 'template_name'
TAGS_DS_TAG_NAME_COLUMN_LABEL = 'tag_name'

# Expected order for the CSV header columns, Tags Table.
TAGS_DS_COLUMNS_ORDER = (TAGS_DS_LINKED_RESOURCE_COLUMN_LABEL, TAGS_DS_TEMPLATE_NAME_COLUMN_LABEL,
                         TAGS_DS_SCHEMA_COLUMN_COLUMN_LABEL, TAGS_DS_FIELD_ID_COLUMN_LABEL,
                         TAGS_DS_FIELD_VALUE_COLUMN_LABEL)

TAGS_DS_EXPORT_COLUMNS_ORDER = (TAGS_DS_RELATIVE_RESOURCE_NAME_COLUMN_LABEL,
                                TAGS_DS_LINKED_RESOURCE_COLUMN_LABEL,
                                TAGS_DS_TEMPLATE_NAME_COLUMN_LABEL, TAGS_DS_TAG_NAME_COLUMN_LABEL,
                                TAGS_DS_SCHEMA_COLUMN_COLUMN_LABEL, TAGS_DS_FIELD_ID_COLUMN_LABEL,
                                TAGS_DS_FIELD_TYPE_COLUMN_LABEL, TAGS_DS_FIELD_VALUE_COLUMN_LABEL)

# Columns that can be empty on the CSV, Tags Table.
TAGS_DS_FILLABLE_COLUMNS = [
    TAGS_DS_LINKED_RESOURCE_COLUMN_LABEL, TAGS_DS_TEMPLATE_NAME_COLUMN_LABEL
]

# Columns that are required on the CSV, Tags Table.
TAGS_DS_NON_FILLABLE_COLUMNS = [
    TAGS_DS_SCHEMA_COLUMN_COLUMN_LABEL, TAGS_DS_FIELD_ID_COLUMN_LABEL,
    TAGS_DS_FIELD_VALUE_COLUMN_LABEL
]

# Constants used for creating the Tag Template used to tag the Datasource, Tag Template Table.
TAG_TEMPLATES_DS_TEMPLATE_NAME_COLUMN_LABEL = 'template_name'
TAG_TEMPLATES_DS_TEMPLATE_DISPLAY_NAME_COLUMN_LABEL = 'display_name'
TAG_TEMPLATES_DS_FIELD_ID_COLUMN_LABEL = 'field_id'
TAG_TEMPLATES_DS_FIELD_DISPLAY_NAME_COLUMN_LABEL = 'field_display_name'
TAG_TEMPLATES_DS_FIELD_TYPE_COLUMN_LABEL = 'field_type'
TAG_TEMPLATES_DS_FIELD_ENUM_VALUES_COLUMN_LABEL = 'enum_values'

# Expected order for the CSV header columns, Tag Template Table.
TAG_TEMPLATES_DS_COLUMNS_ORDER = (TAG_TEMPLATES_DS_TEMPLATE_NAME_COLUMN_LABEL,
                                  TAG_TEMPLATES_DS_TEMPLATE_DISPLAY_NAME_COLUMN_LABEL,
                                  TAG_TEMPLATES_DS_FIELD_ID_COLUMN_LABEL,
                                  TAG_TEMPLATES_DS_FIELD_DISPLAY_NAME_COLUMN_LABEL,
                                  TAG_TEMPLATES_DS_FIELD_TYPE_COLUMN_LABEL,
                                  TAG_TEMPLATES_DS_FIELD_ENUM_VALUES_COLUMN_LABEL)

# Columns that can be empty on the CSV, Tag Template Table.
TAG_TEMPLATES_DS_FILLABLE_COLUMNS = [
    TAG_TEMPLATES_DS_TEMPLATE_NAME_COLUMN_LABEL,
    TAG_TEMPLATES_DS_TEMPLATE_DISPLAY_NAME_COLUMN_LABEL
]

# Columns that are required on the CSV, Tag Template Table.
TAG_TEMPLATES_DS_NON_FILLABLE_COLUMNS = [
    TAGS_DS_FIELD_ID_COLUMN_LABEL, TAG_TEMPLATES_DS_FIELD_DISPLAY_NAME_COLUMN_LABEL,
    TAG_TEMPLATES_DS_FIELD_TYPE_COLUMN_LABEL, TAG_TEMPLATES_DS_FIELD_ENUM_VALUES_COLUMN_LABEL
]

# Value used to split the values inside TAG_TEMPLATES_DS_FIELD_ENUM_VALUES_COLUMN_LABEL field.
ENUM_VALUES_SEPARATOR = "|"

# Constants used for exporting tags summary.
TAGS_DS_SUMMARY_TAG_TEMPLATE_NAME = 'template_name'
TAGS_DS_SUMMARY_TAGS_COUNT = 'tags_count'
TAGS_DS_SUMMARY_TAGGED_ENTRIES_COUNT = 'tagged_entries_count'
TAGS_DS_SUMMARY_TAGGED_COLUMNS_COUNT = 'tagged_columns_count'
TAGS_DS_SUMMARY_TAG_STRING_FIELDS_COUNT = 'tag_string_fields_count'
TAGS_DS_SUMMARY_TAG_BOOL_FIELDS_COUNT = 'tag_bool_fields_count'
TAGS_DS_SUMMARY_TAG_DOUBLE_FIELDS_COUNT = 'tag_double_fields_count'
TAGS_DS_SUMMARY_TAG_TIMESTAMP_FIELDS_COUNT = 'tag_timestamp_fields_count'
TAGS_DS_SUMMARY_TAG_ENUM_FIELDS_COUNT = 'tag_enum_fields_count'

# Expected order for the CSV header columns, Tag Summary Table.
TAGS_DS_SUMMARY_COLUMNS_ORDER = (TAGS_DS_SUMMARY_TAG_TEMPLATE_NAME, TAGS_DS_SUMMARY_TAGS_COUNT,
                                 TAGS_DS_SUMMARY_TAGGED_ENTRIES_COUNT,
                                 TAGS_DS_SUMMARY_TAGGED_COLUMNS_COUNT,
                                 TAGS_DS_SUMMARY_TAG_STRING_FIELDS_COUNT,
                                 TAGS_DS_SUMMARY_TAG_BOOL_FIELDS_COUNT,
                                 TAGS_DS_SUMMARY_TAG_DOUBLE_FIELDS_COUNT,
                                 TAGS_DS_SUMMARY_TAG_TIMESTAMP_FIELDS_COUNT,
                                 TAGS_DS_SUMMARY_TAG_ENUM_FIELDS_COUNT)



import logging
import os

from google.api_core import exceptions
from google.protobuf.json_format import MessageToDict
import pandas as pd
from tabulate import tabulate

from . import constant, datacatalog_facade


class TagDatasourceExporter:

    def __init__(self):
        self.__datacatalog_facade = datacatalog_facade.DataCatalogFacade()

    def export_tags(self, project_ids, dir_path=None):
        """
        Export Tags found by searching Data Catalog.

        :param dir_path: Directory path to be exported to.
        :param project_ids: Project ids to narrow down search results.
        """
        logging.info('')
        logging.info('===> Export Tags [STARTED]')

        logging.info('')
        logging.info('Exporting the Tags...')
        self.__export_tags(project_ids, dir_path)

        logging.info('')
        logging.info('==== Export Tags [FINISHED] =============')

    def __export_tags(self, project_ids, dir_path=None):
        search_results = self.__datacatalog_facade.search_tag_templates(project_ids)

        if dir_path is None:
            dir_path = os.path.dirname(os.path.abspath(__file__))

        summary_dataframe = None

        for search_result in search_results:
            template_dataframe = None
            tag_template_name = search_result.relative_resource_name

            project_id, location_id, tag_template_id = \
                self.__datacatalog_facade.extract_resources_from_template(tag_template_name)

            logging.info('')
            logging.info('Looking for Tags from Template: {}...'.format(tag_template_id))

            tagged_assets = self.__datacatalog_facade.search_tagged_assets(
                project_id, tag_template_id)

            entry_name = ''
            for tagged_asset in tagged_assets:
                try:
                    entry_name = tagged_asset.relative_resource_name

                    logging.info('Loading Tags from Entry: {}...'.format(entry_name))

                    linked_resource = tagged_asset.linked_resource
                    relative_resource_name = tagged_asset.relative_resource_name

                    tags = self.__datacatalog_facade.list_tags(entry_name)
                    asset_dataframe = self.__tags_to_dataframe(linked_resource,
                                                               relative_resource_name,
                                                               tag_template_name, tags)

                    if template_dataframe is not None:
                        template_dataframe = template_dataframe.append(asset_dataframe)
                    else:
                        template_dataframe = asset_dataframe
                except exceptions.PermissionDenied:
                    logging.warning(
                        'Permission denied when processing up Entry %s.'
                        ' The resource will be skipped.', entry_name)

            if template_dataframe is not None:
                file_path = os.path.join(dir_path, '{}.csv'.format(tag_template_id))
                template_dataframe.to_csv(file_path)
                logging.info('==> Tags from Template: {} exported.'.format(tag_template_id))

                if summary_dataframe is not None:
                    summary_dataframe = summary_dataframe.append(template_dataframe)
                else:
                    summary_dataframe = template_dataframe
            else:
                template_dataframe = self.__create_empty_dataframe(tag_template_name)
                logging.info('No Tags found for Template: {}.'.format(tag_template_id))
                if summary_dataframe is not None:
                    summary_dataframe = summary_dataframe.append(template_dataframe)
                else:
                    summary_dataframe = template_dataframe

        if summary_dataframe is not None:
            summary_dataframe.set_index(constant.TAGS_DS_TEMPLATE_NAME_COLUMN_LABEL, inplace=True)
            summary_dataframe_with_stats = self.__create_summary_file(summary_dataframe)

            file_path = os.path.join(dir_path, 'summary.csv')
            summary_dataframe_with_stats.to_csv(file_path)

            logging.info('')
            logging.info('SUMMARY TABLE')
            pd.set_option('display.max_rows', 1000)
            logging.info('\n {}'.format(
                tabulate(summary_dataframe_with_stats, headers='keys', tablefmt='psql')))
            logging.info('')
            logging.info('Check the generated summary file at: %s', file_path)
            logging.info('Check additional files for templates with tags at: %s', dir_path)

    @classmethod
    def __create_summary_file(cls, summary_dataframe):
        dataframe = pd.DataFrame(columns=constant.TAGS_DS_SUMMARY_COLUMNS_ORDER)

        key_values = summary_dataframe.index.unique().tolist()
        for key_value in key_values:
            # We use an array with: [key_value] to make sure the dataframe loc
            # always returns a dataframe, and not a Series
            template_subset = summary_dataframe.loc[[key_value]]
            summary_dataframe.drop(key_value, inplace=True)

            tag_template_name = key_value
            tags_count = template_subset['tag_name'][template_subset['tag_name'] != ''].nunique()
            tagged_columns_count = template_subset['column'][
                template_subset['column'] != ''].nunique()
            tagged_entries_count = tags_count - tagged_columns_count
            field_type_count = template_subset['field_type'].value_counts()

            string_fields_count = 0
            bool_fields_count = 0
            double_fields_count = 0
            timestamp_fields_count = 0
            enum_fields_count = 0

            for key, value in field_type_count.iteritems():
                if key == 'STRING':
                    string_fields_count = value
                elif key == 'BOOL':
                    bool_fields_count = value
                elif key == 'DOUBLE':
                    double_fields_count = value
                elif key == 'TIMESTAMP':
                    timestamp_fields_count = value
                elif key == 'ENUM':
                    enum_fields_count = value

            dataframe = dataframe.append(
                {
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[0]: tag_template_name,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[1]: tags_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[2]: tagged_entries_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[3]: tagged_columns_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[4]: string_fields_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[5]: bool_fields_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[6]: double_fields_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[7]: timestamp_fields_count,
                    constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[8]: enum_fields_count
                },
                ignore_index=True)

        dataframe.set_index(constant.TAGS_DS_SUMMARY_COLUMNS_ORDER[0], inplace=True)
        return dataframe

    @classmethod
    def __tags_to_dataframe(cls, linked_resource, relative_resource_name, tag_template_name, tags):
        dataframe = pd.DataFrame(columns=constant.TAGS_DS_EXPORT_COLUMNS_ORDER)

        for tag in tags:
            current_tag_template_name = tag.template
            if tag_template_name == current_tag_template_name:
                tag_name = tag.name
                column = tag.column
                fields = tag.fields

                for field_id, field_values in fields.items():
                    # We must convert the message to a dict,
                    # otherwise the protobuf message initialize the types
                    # and we are not able to verify what type the field is.
                    field_values_dict = MessageToDict(field_values)

                    string_value = field_values_dict.get('stringValue')
                    bool_value = field_values_dict.get('boolValue')
                    double_value = field_values_dict.get('doubleValue')
                    timestamp_value = field_values_dict.get('timestampValue')
                    enum_value = field_values_dict.get('enumValue', {}).get('displayName')

                    field_type = None
                    field_value = None

                    if string_value is not None:
                        field_value = string_value
                        field_type = 'STRING'
                    elif bool_value is not None:
                        field_value = bool_value
                        field_type = 'BOOL'
                    elif double_value is not None:
                        field_value = double_value
                        field_type = 'DOUBLE'
                    elif timestamp_value is not None:
                        field_value = timestamp_value
                        field_type = 'TIMESTAMP'
                    elif enum_value is not None:
                        field_value = enum_value
                        field_type = 'ENUM'

                    field_index = list(fields.keys()).index(field_id)

                    if field_index == 0:
                        dataframe = dataframe.append(
                            {
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[0]: relative_resource_name,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[1]: linked_resource,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[2]: tag_template_name,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[3]: tag_name,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[4]: column,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[5]: field_id,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[6]: field_type,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[7]: field_value
                            },
                            ignore_index=True)
                    else:
                        dataframe = dataframe.append(
                            {
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[0]: '',
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[1]: linked_resource,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[2]: tag_template_name,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[3]: tag_name,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[4]: column,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[5]: field_id,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[6]: field_type,
                                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[7]: field_value
                            },
                            ignore_index=True)

        dataframe.set_index(constant.TAGS_DS_RELATIVE_RESOURCE_NAME_COLUMN_LABEL, inplace=True)

        return dataframe

    @classmethod
    def __create_empty_dataframe(cls, tag_template_name):
        dataframe = pd.DataFrame(columns=constant.TAGS_DS_EXPORT_COLUMNS_ORDER)

        dataframe = dataframe.append(
            {
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[0]: 'EMPTY_RESOURCE',
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[1]: '',
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[2]: tag_template_name,
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[3]: '',
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[4]: '',
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[5]: '',
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[6]: '',
                constant.TAGS_DS_EXPORT_COLUMNS_ORDER[7]: ''
            },
            ignore_index=True)

        dataframe.set_index(constant.TAGS_DS_RELATIVE_RESOURCE_NAME_COLUMN_LABEL, inplace=True)

        return dataframe



import logging

from google.api_core import exceptions
import pandas as pd

from . import constant, datacatalog_entity_factory, datacatalog_facade


class TagTemplateDatasourceProcessor:

    def __init__(self):
        self.__datacatalog_facade = datacatalog_facade.DataCatalogFacade()

    def create_tag_templates_from_csv(self, file_path):
        """
        Create Tag Templates by reading information from a CSV file.

        :param file_path: The CSV file path.
        :return: A list with all Tag Templates created.
        """
        logging.info('')
        logging.info('===> Create Tag Templates from CSV [STARTED]')

        logging.info('')
        logging.info('Reading CSV file: %s...', file_path)
        dataframe = pd.read_csv(file_path)

        logging.info('')
        logging.info(f'Creating the Tag Templates...')
        created_tag_templates = self.__create_tag_templates_from_dataframe(dataframe)

        logging.info('')
        logging.info('==== Create Tag Templates from CSV [FINISHED] =============')

        return created_tag_templates

    def delete_tag_templates_from_csv(self, file_path):
        """
        Delete Tag Templates by reading information from a CSV file.

        :param file_path: The CSV file path.
        """
        logging.info('')
        logging.info('===> Delete Tag Templates from CSV [STARTED]')

        logging.info('')
        logging.info('Reading CSV file: %s...', file_path)
        dataframe = pd.read_csv(file_path)

        logging.info('')
        logging.info('Deleting the Tag Templates...')
        self.__delete_tag_templates_from_dataframe(dataframe)

        logging.info('')
        logging.info('==== Delete Tag Templates from CSV [FINISHED] =============')

    def __create_tag_templates_from_dataframe(self, dataframe):
        normalized_df = self.__normalize_dataframe(dataframe)
        normalized_df.set_index(constant.TAG_TEMPLATES_DS_TEMPLATE_NAME_COLUMN_LABEL, inplace=True)

        created_tag_templates = []
        for tag_template_name in normalized_df.index.unique().tolist():

            template_subset = normalized_df.loc[[tag_template_name]]

            # Save memory by deleting data already copied to a subset.
            normalized_df.drop(tag_template_name, inplace=True)

            tag_template = self.__create_tag_template_from_dataframe(template_subset)

            project_id, location_id, tag_template_id = \
                self.__datacatalog_facade.extract_resources_from_template(tag_template_name)

            try:
                created_tag_template = self.__datacatalog_facade.create_tag_template(
                    project_id, location_id, tag_template_id, tag_template)
                created_tag_templates.extend([created_tag_template])
                logging.info('Template %s created.', tag_template_name)
            except exceptions.AlreadyExists:
                logging.warning('Template %s already exists.', tag_template_name)

        return created_tag_templates

    def __delete_tag_templates_from_dataframe(self, dataframe):
        normalized_df = self.__normalize_dataframe(dataframe)
        normalized_df.set_index(constant.TAG_TEMPLATES_DS_TEMPLATE_NAME_COLUMN_LABEL, inplace=True)

        for tag_template_name in normalized_df.index.unique().tolist():

            try:
                self.__datacatalog_facade.delete_tag_template(tag_template_name)
                logging.info('Template %s deleted.', tag_template_name)
            except exceptions.GoogleAPICallError as e:
                logging.warning('Exception deleting template %s.: %s', tag_template_name, str(e))

    @classmethod
    def __normalize_dataframe(cls, dataframe):
        # Reorder dataframe columns.
        ordered_df = dataframe.reindex(columns=constant.TAG_TEMPLATES_DS_COLUMNS_ORDER, copy=False)

        # Fill NA/NaN values by propagating the last valid observation forward to next valid.
        filled_subset = ordered_df[constant.TAG_TEMPLATES_DS_FILLABLE_COLUMNS].fillna(method='pad')

        # Rebuild the dataframe by concatenating the fillable and non-fillable columns.
        rebuilt_df = pd.concat(
            [filled_subset, ordered_df[constant.TAG_TEMPLATES_DS_NON_FILLABLE_COLUMNS]], axis=1)

        return rebuilt_df

    def __create_tag_template_from_dataframe(self, dataframe):
        template_name = dataframe.index.unique().tolist()[0]

        fields_subset = \
            dataframe.loc[[template_name], constant.TAG_TEMPLATES_DS_FIELD_ID_COLUMN_LABEL:]

        fields_dict = self.__convert_fields_dataframe_to_dict(fields_subset)

        tag_template_dict = {
            'name': template_name,
            'display_name':
            dataframe[constant.TAG_TEMPLATES_DS_TEMPLATE_DISPLAY_NAME_COLUMN_LABEL][0],
            'fields': fields_dict
        }

        return self.__create_tag_template_from_dict(tag_template_dict)

    @classmethod
    def __create_tag_template_from_dict(cls, tag_template_dict):
        return datacatalog_entity_factory.DataCatalogEntityFactory.make_tag_template(
            tag_template_dict)

    @classmethod
    def __convert_fields_dataframe_to_dict(cls, dataframe):
        base_dict = dataframe.to_dict(orient='records')

        id_to_field_map = {}
        for base_object in base_dict:
            enum_values = base_object[constant.TAG_TEMPLATES_DS_FIELD_ENUM_VALUES_COLUMN_LABEL]

            if not isinstance(enum_values, str):
                enum_values = ''

            id_to_field_map[base_object[constant.TAG_TEMPLATES_DS_FIELD_ID_COLUMN_LABEL]] = \
                {
                    constant.TAG_TEMPLATES_DS_FIELD_DISPLAY_NAME_COLUMN_LABEL:
                        base_object[constant.TAG_TEMPLATES_DS_FIELD_DISPLAY_NAME_COLUMN_LABEL],
                    constant.TAG_TEMPLATES_DS_FIELD_TYPE_COLUMN_LABEL:
                        base_object[constant.TAG_TEMPLATES_DS_FIELD_TYPE_COLUMN_LABEL],
                    constant.TAG_TEMPLATES_DS_FIELD_ENUM_VALUES_COLUMN_LABEL:
                        enum_values.split(
                            constant.ENUM_VALUES_SEPARATOR)
                }

        return id_to_field_map



import setuptools

with open('README.md') as readme_file:
    readme = readme_file.read()

with open('HISTORY.md') as history_file:
    history = history_file.read()

setuptools.setup(
    name='datacatalog-util',
    version='0.2.0',
    author='Marcelo Miranda',
    author_email='mesmacosta@gmail.com',
    description='A package to manage Google Cloud Data Catalog'
    ' helper commands and scripts',
    platforms='Posix; MacOS X; Windows',
    packages=setuptools.find_packages(where='./src'),
    package_dir={'': 'src'},
    entry_points={
        'console_scripts': [
            'datacatalog-util = datacatalog_util:main',
        ],
    },
    include_package_data=True,
    install_requires=(
        'google-cloud-datacatalog',
        'pandas',
        'tabulate',
        'datacatalog-tag-manager',
        'datacatalog-fileset-enricher'
    ),
    setup_requires=('pytest-runner', ),
    tests_require=('pytest-cov', ),
    python_requires='>=3.6',
    license="MIT license",
    long_description=readme + '\n\n' + history,
    long_description_content_type='text/markdown',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
    ],
    url='https://github.com/mesmacosta/datacatalog-util',
    zip_safe=False,
)



import unittest
from unittest import mock

from google.api_core import exceptions
from google.cloud.datacatalog import types

from datacatalog_util import datacatalog_facade


class DataCatalogFacadeTest(unittest.TestCase):

    @mock.patch('datacatalog_util.datacatalog_facade.datacatalog.DataCatalogClient')
    def setUp(self, mock_datacatalog_client):
        self.__datacatalog_facade = datacatalog_facade.DataCatalogFacade()
        # Shortcut for the object assigned to self.__datacatalog_facade.__datacatalog
        self.__datacatalog_client = mock_datacatalog_client.return_value

    def test_constructor_should_set_instance_attributes(self):
        self.assertIsNotNone(self.__datacatalog_facade.__dict__['_DataCatalogFacade__datacatalog'])

    def test_create_tag_template_should_succeed(self):
        self.__datacatalog_facade.create_tag_template('test-project', 'location-id',
                                                      'tag_template_id', {})

        datacatalog = self.__datacatalog_client
        self.assertEqual(1, datacatalog.create_tag_template.call_count)

    def test_delete_tag_template_should_succeed(self):
        self.__datacatalog_facade.delete_tag_template('tag_template_name')

        datacatalog = self.__datacatalog_client
        self.assertEqual(1, datacatalog.delete_tag_template.call_count)

    def test_get_tag_template_should_call_client_library_method(self):
        self.__datacatalog_facade.get_tag_template(None)

        datacatalog = self.__datacatalog_client
        datacatalog.get_tag_template.assert_called_once()

    def test_search_tag_templates_should_return_values(self):
        result_iterator = MockedObject()

        entry = MockedObject()
        entry.name = 'template_1'

        entry_2 = MockedObject()
        entry_2.name = 'template_2'

        expected_return_value = [entry, entry_2]

        # simulates two pages
        result_iterator.pages = [[entry], [entry_2]]

        datacatalog = self.__datacatalog_client
        datacatalog.search_catalog.return_value = result_iterator

        return_value = self.__datacatalog_facade.search_tag_templates('my-project1,my-project2')

        self.assertEqual(1, datacatalog.search_catalog.call_count)
        self.assertEqual(expected_return_value, return_value)

    def test_search_tagged_assets_should_return_values(self):
        result_iterator = MockedObject()

        entry = MockedObject()
        entry.name = 'asset_1'

        entry_2 = MockedObject()
        entry_2.name = 'asset_2'

        expected_return_value = [entry, entry_2]

        # simulates two pages
        result_iterator.pages = [[entry], [entry_2]]

        datacatalog = self.__datacatalog_client
        datacatalog.search_catalog.return_value = result_iterator

        return_value = self.__datacatalog_facade.search_tagged_assets('my-project1', 'template1')

        self.assertEqual(1, datacatalog.search_catalog.call_count)
        self.assertEqual(expected_return_value, return_value)

    def test_get_tag_templates_from_search_results_should_return_values(self):
        entry = MockedObject()
        entry.name = 'asset_1'
        entry.relative_resource_name = 'asset_1_resource_name'

        entry_2 = MockedObject()
        entry_2.name = 'asset_2'
        entry_2.relative_resource_name = 'asset_2_resource_name'

        search_results = [entry, entry_2]

        datacatalog = self.__datacatalog_client
        datacatalog.get_tag_template.return_value = {}

        return_value = self.__datacatalog_facade.get_tag_templates_from_search_results(
            search_results)

        self.assertEqual(2, datacatalog.get_tag_template.call_count)
        self.assertEqual([{}, {}], return_value)

    def test_get_tag_templates_err_from_search_results_should_return_values(self):
        entry = MockedObject()
        entry.name = 'asset_1'
        entry.relative_resource_name = 'asset_1_resource_name'

        entry_2 = MockedObject()
        entry_2.name = 'asset_2'
        entry_2.relative_resource_name = 'asset_2_resource_name'

        search_results = [entry, entry_2]

        datacatalog = self.__datacatalog_client
        datacatalog.get_tag_template.side_effect = [
            {}, exceptions.GoogleAPICallError('Permission denied')
        ]

        return_value = self.__datacatalog_facade.get_tag_templates_from_search_results(
            search_results)

        self.assertEqual(2, datacatalog.get_tag_template.call_count)
        self.assertEqual([{}], return_value)

    def test_extract_resources_from_template_should_return_values(self):
        resource_name = 'projects/my-project/locations/us-central1/tagTemplates/my-template'

        project_id, location_id, tag_template_id = \
            self.__datacatalog_facade.extract_resources_from_template(resource_name)

        self.assertEqual('my-project', project_id)
        self.assertEqual('us-central1', location_id)
        self.assertEqual('my-template', tag_template_id)


class MockedObject(object):

    def __setitem__(self, key, value):
        self.__dict__[key] = value

    def __getitem__(self, key):
        return self.__dict__[key]


def make_fake_tag():
    tag = types.Tag()
    tag.template = 'test_template'
    tag.fields['test_bool_field'].bool_value = True
    tag.fields['test_double_field'].double_value = 1
    tag.fields['test_string_field'].string_value = 'Test String Value'
    tag.fields['test_timestamp_field'].timestamp_value.FromJsonString('2019-10-15T01:00:00-03:00')
    tag.fields['test_enum_field'].enum_value.display_name = 'Test ENUM Value'

    return tag
